unit LD097u1;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, FileUtil, Forms, Controls, Graphics, Dialogs, StdCtrls;

const ver='2 Aug 2011';
//Started 2 Aug 11... my first "serious" Delphi->Lazarus conversion

type

  { TLD097f1 }

  TLD097f1 = class(TForm)
    buQuit: TButton;
    buSpecSource: TButton;
    buDest: TButton;
    buDoLoadCopySave: TButton;
    laExplainLock: TLabel;
    laInst2: TLabel;
    laInst1: TLabel;
    laDest: TLabel;
    laSource: TLabel;
    meSource: TMemo;
    meDest: TMemo;
    OpenDialog1: TOpenDialog;
    SaveDialog1: TSaveDialog;
    procedure buDestClick(Sender: TObject);
    procedure buDoLoadCopySaveClick(Sender: TObject);
    procedure buQuitClick(Sender: TObject);
    procedure buSpecSourceClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
  private
    { private declarations }
    sTmp,sTmp2,sPathNNameSource,sPathNNameDest:string;
    boSpecdSource,boSpecdDest:boolean;
    procedure CopyBetweenMemos;
  public
    { public declarations }
  end; 

var
  LD097f1: TLD097f1;

implementation

{$R *.lfm}

{ TLD097f1 }

procedure TLD097f1.FormCreate(Sender: TObject);
begin
  (*Next two lines cause application to open in a "sensible" position
  regardless of the resolution the user's screen is set to.
  All of the properties referenced, e.g. screen.height, are built
  in to the ordinary Delphi environment.

  Having these two lines in the code means that the form can be
  wherever you find it most convenient on your desktop while
  developing the application*)
top:=(screen.height div 3)-(clientheight div 2);
left:=(screen.width div 2)-(clientwidth div 2);

application.title:='DD97';//Label on icon, taskbar button
caption:='DD97, version '+ver;//Title on window
laSource.caption:='(source file not yet specified)';
laDest.caption:='(destination file not yet specified)';
laExplainLock.caption:='You must specify source and destination files '+
   'before you can click "Do it".';
buDoLoadCopySave.enabled:=false;

boSpecdSource:=false;
boSpecdDest:=false;

(*Next... configure the two memos as needed for our wants.
  One memo holds the text as it was in the source file
  The second will be filled by the application from the
  text in the first. Hence the memo's names: meSource, meDest.
Most of the memo properties can be as set by default by Delphi,
  here you see the non-default choices you need....

meSource.:=false;
meDest.:=false;
*)

meSource.WantReturns:=false;
meDest.WantReturns:=false;

meSource.wordwrap:=false;
meDest.wordwrap:=false;

meSource.scrollbars:=ssBoth;//not "necessary".. but could be useful
meDest.scrollbars:=ssBoth;

meSource.readonly:=true;//not "necessary".. but will prevent accidents

meSource.tabstop:=false;
meDest.tabstop:=false;

end;

procedure TLD097f1.buQuitClick(Sender: TObject);
begin
  application.terminate;//"close" may be better?
end;

procedure TLD097f1.buDestClick(Sender: TObject);
(*There are all sorts of things you can do to "tailor"
users' experiences with the SaveDialog. For instance,
you can set things up to warn users if they are
starting down a course of action which will
cause an existing file to be overwritten.
Those "frills" I am leaving to another time.*)
begin
if SaveDialog1.execute then begin
   sPathNNameDest:=SaveDialog1.filename;
   laDest.caption:=sPathNNameDest;
   boSpecdDest:=true;
   if (boSpecdSource) and (boSpecdDest) then buDoLoadCopySave.enabled:=true;
   end;
end;

procedure TLD097f1.buDoLoadCopySaveClick(Sender: TObject);
begin
meSource.lines.loadfromfile(sPathNNameSource);
CopyBetweenMemos;
meDest.lines.savetofile(sPathNNameDest);
end;


procedure TLD097f1.buSpecSourceClick(Sender: TObject);
(*There are all sorts of things you can do to "tailor"
users' experiences with the OpenDialog. For instance,
you can set the filter to show only files with the
extension or extensions you want them to see.
Those "frills" I am leaving to another time.*)
begin
if OpenDialog1.execute then begin
   sPathNNameSource:=OpenDialog1.filename;
   laSource.caption:=sPathNNameSource;
   boSpecdSource:=true;
   if (boSpecdSource) and (boSpecdDest) then buDoLoadCopySave.enabled:=true;
   end;
end;

procedure TLD097f1.CopyBetweenMemos;
(*This is the "heart" of the program, in terms of
it doing something useful.

For this tutorial, all it is doing is copying the
text from the source file to the destination.

For a simple copy, all that would be needed is...

meDest.clear;//clear it; prepare for filling
for wWhatLine:=0 to meSource.lines.count-1 do begin
   sTmp:=meSource.lines[wWhatLine];
   meDest.lines.add(sTmp);
   end;//for loop

... however that would be REALLY pointless, so for
the sake of the tutorial, we will dress it up a
little and make the following "just for demonstration"
changes to the text as it is copied. We will be...

1) substituting 'X' each time an 'n' is encountered, and
2) dropping all 'a's out of the text.

... which is a pointless exercise. But I hope it
shows you the basics of whatever transformation
you want to make to your files? In all of the above,
I was speaking in case sensitive terms.

No check, by the way, is made to see that the file
only holds printable characters.

In some ways, that won't matter, but you'll have
trouble looking at the file with some text
editors if the characters in the file include
non-printing characters.
*)
var wWhatLine,c1:word;

begin
meDest.clear;//clear it; prepare for filling
for wWhatLine:=0 to meSource.lines.count-1 do begin
   sTmp:=meSource.lines[wWhatLine];
   sTmp2:='';
   for c1:=1 to length(sTmp) do begin
       if sTmp[c1]='n' then sTmp2:=sTmp2+'X';
       if (sTmp[c1]<>'a') and (sTmp[c1]<>'n') then sTmp2:=sTmp2+sTmp[c1];
       (*if sTmp[c1]='a' then do nothing*)
       end;//for c1 loop
   meDest.lines.add(sTmp2);
   end;//for wWhatLine loop
end;

end.

